#import "Controller.h"
#import "NSImage+TIFFCombiner.h"


@implementation Controller


#define convert( from, to) \
  [image ## to ## View setImage:[[image ## from ## View image] resizedImageWithSize:[image ## to ## View frame].size]];


- (void) convert32to16:(id)sender
{
   convert( 32, 16);
}


- (void) convert48to24:(id)sender
{
   convert( 48, 24);
}


- (void) convert48to32:(id)sender
{
   convert( 48, 32);
}


- (void) convert64to48:(id)sender
{
   convert( 64, 48);
}


- (void) convert128to32:(id)sender
{
   convert( 128, 32);
}


- (void) convert128to48:(id)sender
{
   convert( 128, 48);
}


- (void) convert128to64:(id)sender
{
   convert( 128, 64);
}



- (void) saveAsData:(NSData *) data
{
   NSSavePanel   *panel;
   NSString	 *dir;

   if( ! (dir = [[NSUserDefaults standardUserDefaults] stringForKey:@"savePath"]))
      dir = NSHomeDirectory();
   panel = [NSSavePanel savePanel];

   if( NSOKButton == [panel runModalForDirectory:dir
                                            file:@"icon.tiff"])
   {
      NSString  *s;

      s = [panel filename];
      [[NSUserDefaults standardUserDefaults] setObject:[s stringByDeletingLastPathComponent]
                                                   forKey:@"savePath"];
      [data writeToFile:s
             atomically:YES];
   }
}



//
// Keep only those representations the user selected
// some of the TIFFs might still have multiple (unwanted)
// representations. Also ensure that all reps are square!
// At this point we still add duplicates.
//
- (void) addWantedRepresentations:(NSArray *) representations
                          toArray:(NSMutableArray *) array
{
   unsigned int  n;
   int           i;
   NSImageRep    *p;
   unsigned int  wh;
   
   if( ! (n = [representations count]))
      return;
   
   for( i = n; --i >= 0;)
   {
      p = [representations objectAtIndex:i];
      if( (wh = [p size].width) == [p size].height)
      {
         if( (wh == 16 && [image16View image]) ||
             (wh == 24 && [image24View image]) ||
             (wh == 32 && [image32View image]) ||
             (wh == 48 && [image48View image]) ||
             (wh == 64 && [image64View image]) ||
             (wh == 128 && [image128View image]))
         {
            [array addObject:p];
         }
      }
   }
}


//
// Collect representations 
//
- (NSArray *) collectRepresentations
{
   NSMutableArray   *array;
   
   array = [NSMutableArray arrayWithCapacity:32];

   [self addWantedRepresentations:[[image16View image] representations]
                          toArray:array];
   [self addWantedRepresentations:[[image24View image] representations]
                          toArray:array];
   [self addWantedRepresentations:[[image32View image] representations]
                          toArray:array];
   [self addWantedRepresentations:[[image48View image] representations]
                          toArray:array];
   [self addWantedRepresentations:[[image64View image] representations]
                          toArray:array];
   [self addWantedRepresentations:[[image128View image] representations]
                          toArray:array];
   return( array);
}


//
// Collect representations from Icons and save'em
//
- (void) saveDocumentAs:(id)sender
{
   NSArray          *representations;
   NSMutableArray   *keepers;
   NSImage          *image;
   unsigned int     n;
   int              i;
   int              j;
   NSImageRep       *p;
   NSImageRep       *q;
   NSSize           maxSize;
   
   
   representations = [self collectRepresentations];

   if( ! (n = [representations count]))
   {
      NSBeep();
      return;
   }


//
// Now sort out duplicates, we keep only the best
// quadratic algorithm... 
//
   keepers = [NSMutableArray arrayWithCapacity:n];
   maxSize = NSMakeSize( 0, 0);
   
   for( i = n; --i >= 0;)
   {
      p = [representations objectAtIndex:i];
      for( j = [keepers count]; --j >= 0;)
      {
         q = [keepers objectAtIndex:j];
         if( [p size].width == [q size].width)
         {
            if( [p bitsPerSample] > [q bitsPerSample])
            {
               [keepers replaceObjectAtIndex:j
                                  withObject:[[p copy] autorelease]];
               if( [p size].width > maxSize.width)
                  maxSize = [p size];
            }
            goto next;
         }
      }
      [keepers addObject:[[p copy] autorelease]];
      if( [p size].width > maxSize.width)
         maxSize = [p size];
next:;         
   }
   
//
// Now create a new NSImage with our representations
// (size is bogus here!)
//
   image = [[[NSImage alloc] initWithSize:NSMakeSize( 48, 48)] autorelease]; // 48x48 is more convenient
   [image addRepresentations:keepers];

//
// Open save as panel and save as TIFF it...
//
   [self saveAsData:[image TIFFRepresentation]];
}


//
// Implement open routine, so that we can do some nice things automatically
// like a) autowipe
//      b) autoinsert images into views, with proper representations
//      c) if nothing fits put image autoscaled into 128x128
//
- (void) loadImageWithPath:(NSString *) path
{
   NSImage         *image;
   NSArray         *representations;
   NSImageRep      *p;
   BOOL            flag;
   int             i;
   float           wh;
   
   if( ! (image = [[[NSImage alloc] initWithContentsOfFile:path] autorelease]))
      return;

   [self wipe16:self];
   [self wipe24:self];
   [self wipe32:self];
   [self wipe48:self];
   [self wipe64:self];
   [self wipe128:self];

   flag = NO;
   representations = [image representations];
   for( i = [representations count]; --i >= 0;)
   {
      p = [representations objectAtIndex:i];
      if( (wh = [p size].width) == [p size].height)
      {
         if( wh == 16)
         {
            flag = YES;
            [image16View setImage:[NSImage imageWithRepresentation:p]];
         }
         if( wh == 24)
         {
            flag = YES;
            [image24View setImage:[NSImage imageWithRepresentation:p]];
         }
         if( wh == 32)
         {
            flag = YES;
            [image32View setImage:[NSImage imageWithRepresentation:p]];
         }
         if( wh == 48)
         {
            flag = YES;
            [image48View setImage:[NSImage imageWithRepresentation:p]];
         }
         if( wh == 64)
         {
            flag = YES;
            [image64View setImage:[NSImage imageWithRepresentation:p]];
         }
         if( wh == 128)
         {
            flag = YES;
            [image128View setImage:[NSImage imageWithRepresentation:p]];
         }
      }
  }

  if( ! flag)
     [image128View setImage:[image resizedImageWithSize:NSMakeSize( 128, 128)]];
}


- (NSArray *) userSelectFilesFrom:(NSString *) path
                    withExtensions:(NSArray *) extensions
                    allowMultiple:(BOOL) flag
{
   NSOpenPanel   *panel;
   
   if( ! path)
      path = NSHomeDirectory();
   panel = [NSOpenPanel openPanel];
   [panel setAllowsMultipleSelection:flag];
   if( NSOKButton != [panel runModalForDirectory:path
                                            file:nil
                                           types:extensions])
   {
      return( nil);
   }	
   return( [panel filenames]);
}


- (void) openDocument:(id) sender
{
   NSArray   *files;
   NSString  *path;

   path = [[NSUserDefaults standardUserDefaults] stringForKey:@"loadPath"];
   if( files = [self userSelectFilesFrom:path
                          withExtensions:[NSArray arrayWithObjects:@"tiff", @"jpg", @"gif", @"png", nil]
                           allowMultiple:NO])
   {
      if( path = [files lastObject])
      {
         [[NSUserDefaults standardUserDefaults] setObject:[path stringByDeletingLastPathComponent]
                                                   forKey:@"loadPath"];
         [self loadImageWithPath:path];
      }
   }
}

/*#
###
### ERASER METHODS
### 
#*/ 

- (void) wipe16:(id)sender
{
   [image16View setImage:nil];
}


- (void)wipe24:(id)sender
{
   [image24View setImage:nil];
}


- (void)wipe32:(id)sender
{
   [image32View setImage:nil];
}


- (void)wipe48:(id)sender
{
   [image48View setImage:nil];
}


- (void)wipe64:(id)sender
{
   [image64View setImage:nil];
}


- (void)wipe128:(id)sender
{
   [image128View setImage:nil];
}


@end
