/* InfoPanel.m created by znek on Sat 08-Apr-2000 */
/* $Id$
 *
 * $Log$
 */
#import "MKInfoPanel.h"
#import "NSImage+EPS.h"
#import "MKMulleLogoDrawer.h"

#define HZ  	12.5   // up/down this for CPU usage
#define REF_HZ 	10.0   // values coded for 10 Hz


@interface NSClipView(EDKnownPrivate)
- (void) _setDrawsBackground:(BOOL)flag;
@end

@interface NSScrollView(EDKnownPrivate)
- (void) _setDrawsBackground:(BOOL)flag;
@end

#define MIMIC_STANDARD_ABOUT_PANEL  1


static mkZoom zoomTable[] =
{
  { 0.1, 0.05 },
   { 1.1, 0.001 },
//  {0.4, 0.02},
//  {0.8, 0.02},
//  {1.0, 0.01},
//  {1.5, 0.001},
  {1.11, 0.0}
//  {2.0, 0.002},
//  {3.0, 0.0}
};



@implementation MKInfoPanel

+ (NSString *) nibName
{
  return [self description];
}

- (id)init
{
   [super init];
   [NSBundle loadNibNamed:[isa nibName] owner:self];
   useZNeKZoomer = YES;
   hz = HZ;

   return( self);
}


- (void) dealloc
{
  [mulleLogo release];
  #warning (znek) dealloc something
  [super dealloc];
}


- (void) awakeFromNib
{
   NSClipView    *cv;
   NSScrollView  *sv;
   NSRect        frame;

   if(mainWindow == nil)
   {
      NSLog(@"%@: %@ Problem loading nib named %@", isa, NSStringFromSelector(_cmd), [isa nibName]);
      [NSException raise:NSGenericException format:@"that sucks!"];
   }

   cv = (NSClipView *) [textView superview];
   [cv _setDrawsBackground:NO];
   [cv setCopiesOnScroll:NO];

   sv = (NSScrollView *) [cv superview];
   [sv _setDrawsBackground:NO];
   [sv setBorderType:NSNoBorder];

   [textView setDrawsBackground:NO];

   // remember the rects for later use

   insideRect = [sv frame];
   outsideRect = [sv frame];
   outsideRect.origin.x -= 1;
   outsideRect.origin.y -= 1;
   outsideRect.size.width += 2;
   outsideRect.size.height += 2;

   // create alpha mask

   if(1)
   {
      alphaMask = [[NSImage alloc] initWithSize:insideRect.size];

      [alphaMask lockFocus];
      [[NSColor colorWithDeviceWhite:NSWhite alpha:0.6] set];
      NSRectFill(NSMakeRect(0.0, 0.0, [alphaMask size].width, [alphaMask size].height));
      [alphaMask unlockFocus];
   }
   else
   {
      alphaMask = [[NSImage imageNamed:@"MulleKybernetikLogoIPM"] retain];
   }

   // create the shading mask

   shadeMask = [[NSImage alloc] initWithSize:[backgroundImageView frame].size];

   [shadeMask lockFocus];
   [[NSColor colorWithDeviceWhite:NSWhite alpha:0.7] set];
   NSRectFill(NSMakeRect(0.0, 0.0, [shadeMask size].width, [shadeMask size].height));
   [shadeMask unlockFocus];

   // Allocate images
   frame = [backgroundImageView frame];
   bgImage = [[NSImage allocWithZone:[backgroundImageView zone]] initWithSize:frame.size];
   [backgroundImageView setImage:bgImage];

   mulleLogo = [[MKMulleLogoDrawer alloc] init];

   //
   // ensure pic is rectangular
   //
   if( frame.size.height > frame.size.width)
      frame.size.width = frame.size.height;
   frame.size.height = frame.size.width;
   frame.origin = NSZeroPoint;

   panelImageRect = frame;
   panelImage     = [[NSImage allocWithZone:[self zone]] initWithSize:panelImageRect.size];
   [panelImage setBackgroundColor:[NSColor colorWithDeviceWhite:NSLightGray alpha:0.0]];

#if MIMIC_STANDARD_ABOUT_PANEL
   [self updateViewsFromCustominfo];

#endif
}

- (void) updateViewsFromCustominfo
{
   NSBundle       *mainBundle;
   NSString       *path;
   NSDictionary   *customInfo;
   NSString       *s;
   NSString       *string;

   if( mainBundle = [NSBundle mainBundle])
   {
      if( path = [mainBundle pathForResource:@"Credits"
                                      ofType:@"rtf"])
         [textView readRTFDFromFile:path];

      customInfo = [mainBundle infoDictionary];
      if( s = [customInfo objectForKey:@"NSHumanReadableCompleteName"])
         [headerView setStringValue:s];
      if( s = [customInfo objectForKey:@"NSAppVersion"])
         [subHeaderView setStringValue:s];

      s = [customInfo objectForKey:@"NSHumanReadableCopyright"];
      string = [customInfo objectForKey:@"MKHumanReadableInfo"];

      if( s && string)
         s = [NSString stringWithFormat:@"%@\n%@", string, s];
      else
         if( ! s)
            s = string;
      if( s)
         [footerView setStringValue:s];
   }
   else
      NSLog( @"main Bundle not found, strange...!");
   [iconView setImage:[[NSApplication sharedApplication] applicationIconImage]];
}


- (void) updateImageView
{
   NSRect  blitArea;
   NSRect  frame;
   // clear canvas

   frame = [backgroundImageView frame];
   [bgImage lockFocus];

   // clear canvas with that beautiful light gray
   [[NSColor colorWithDeviceWhite:NSLightGray alpha:0.0] set];
   NSRectFill( frame);

   blitArea.origin = NSMakePoint( (panelImageRect.size.width - frame.size.width) / 2,
                                  (panelImageRect.size.height - frame.size.height) / 2);
   blitArea.size   = frame.size;
   [[self panelImage] compositeToPoint:NSZeroPoint
                              fromRect:blitArea
                             operation:NSCompositeSourceOver];


   // shade the image
   [shadeMask compositeToPoint:NSZeroPoint operation:NSCompositeSourceAtop];
   // composite alpha mask into bgImage
   [alphaMask compositeToPoint:insideRect.origin operation:NSCompositeSourceOver];
   // draw frame
   [[NSColor colorWithDeviceWhite:NSBlack alpha:1.0] set];
   NSFrameRect(outsideRect);

   [bgImage unlockFocus];

   [backgroundImageView setNeedsDisplay:YES];
}


//
// a maze of twisty little passages all alike
//
- (NSImage *) panelImage
{
   NSRect              rect;
   NSRect              bounds;
   NSAffineTransform   *transform;
   
   rect.origin   = NSZeroPoint;
   rect.size     = [panelImage size];
   bounds.origin = NSZeroPoint;
   bounds.size   = [mulleLogo size];
   
   [panelImage lockFocus];

   //
   // simulate bounds
   // 
   transform = [NSAffineTransform transform];
   [transform scaleXBy:rect.size.width / bounds.size.width
                   yBy:rect.size.height / bounds.size.height];
   [transform concat];  // reset transform

   [[NSColor lightGrayColor] set];
   NSRectFill( bounds);
   [mulleLogo drawRect:bounds];

   [panelImage unlockFocus];

   return( panelImage);
}


/////////////////////////////////////////////
//
//  ANIMATION
//
/////////////////////////////////////////////


- (void) activateMainWindow:(id)sender
{
  [self startAnimation];
  [self updateImageView];
  [mainWindow center];
  [mainWindow makeKeyAndOrderFront:self];
}


- (void) startAnimation
{
   currentZoom = &zoomTable[ 0];
   [self restartAnimation];
}


- (void) restartAnimation
{
   [timer invalidate];
   [timer release];
   timer = [[NSTimer scheduledTimerWithTimeInterval:1.0 / hz
                                             target:self
                                           selector:@selector( animate:)
                                           userInfo:nil
                                            repeats:YES] retain];
   [mulleLogo setScaling:useZNeKZoomer ? currentZoom->scale :1.0];
   [mulleLogo setRotation:0];
}

- (void) animate:(NSTimer *)timer
{
   if( useZNeKZoomer)
      [self computeNewScale];
   [self computeNewRotation];
   [self updateImageView];
}


- (void) computeNewScale
{
   if( currentZoom->increment > 0.0)
   {
      if( [mulleLogo scaling] >= currentZoom[ 1].scale)
         currentZoom++;
     [mulleLogo incrementScaling:currentZoom->increment * (REF_HZ / hz)];
   }
}


- (void) computeNewRotation
{
   [mulleLogo incrementRotation:-2.5 * (REF_HZ / hz)];
}


- (void) windowWillClose:(NSNotification *)aNotification
{
  [timer invalidate];
  [timer release];
  timer = nil;
}


- (void) setUseZNeKZoomer:(BOOL) flag
{
   useZNeKZoomer = flag;
}


- (void) setHz:(float) value
{
   if( hz != value)
   {
      hz = value;
      if( timer)
      [self restartAnimation];
   }
}
@end
