/* NSImage+TIFFCombiner.m created by nat on Sun 23-Jan-2000 */

#import "NSImage+TIFFCombiner.h"


@implementation NSImage ( TIFFCombiner)


+ (id) imageWithRepresentation:(NSImageRep *) rep
{
   NSImage  *image;
   
   image =  [[[NSImage alloc] initWithSize:[rep size]] autorelease];
   [image addRepresentation:rep];
   return( image);
}


- (BOOL) containsRepresentationWithSize:(NSSize) size
{
   NSEnumerator   *rover;
   NSImageRep     *p;
   NSSize         repSize;
   
   rover = [[self representations] objectEnumerator];

   while( p = [rover nextObject])
   {
      repSize = [p size];
      if( size.width == repSize.width &&
          size.height == repSize.height)
         return( YES);
   }
   return( NO);
}
   

//
// If possible use alternative representations already stored
// in TIFF
//
- (NSImage *) resizedImageWithSize:(NSSize) size
{
   NSImage   *image;
   NSImage   *clone;
   NSRect    rect;
   
   NSAssert( size.width > 0.0 && size.height > 0.0, @"illegal target size");
   NSAssert( [self size].width > 0.0 && [self size].height > 0.0, @"illegal sourcesize");


   if( [self containsRepresentationWithSize:size])
   {
      image = [[self copy] autorelease];
      [image setSize:size];
      return( image);
   }

//
// Create image with properly sized representation
//
   
   image = [[[NSImage alloc] initWithSize:size] autorelease];
   clone = [[self copyWithZone:[self zone]] autorelease];
   [clone setScalesWhenResized:YES];
   [clone setSize:size];

   [image lockFocus];
   [image setFlipped:YES];
   [[NSColor clearColor] set];

   rect.origin = NSMakePoint( 0, 0);
   rect.size   = size;
   NSRectFill( rect);
   [clone compositeToPoint:rect.origin
                 operation:NSCompositeSourceOver];
   [image unlockFocus];

   return( image);
}


@end
